# -*- coding: UTF-8 -*-
# lvjiyong on 2015/6/7.

# 当最大字数少于下述设置时，重新搜索内容
MAX_WORD_RESEARCH = 100

# 如果重新搜索的内容含有下述内容，则重新使用第一次找到内容
NOT_EXIST = [u'免责声明', u'本站注明', u'内容侵权', u'我们联系', u'联系我们', u'版权所有']

IMG_SRC = '(src|srcUrl|data-src|image-src|data-url|image-url|data|image)'

MIN_IMAGE_WIDTH = 200
MIN_IMAGE_HEIGHT = 160
REQUEST_MAX_PAGES = 30

